#---
# Excerpted from "Crafting Rails Applications",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit http://www.pragmaticprogrammer.com/titles/jvrails for more book information.
#---
module MailForm
  class Base
    include ActiveModel::Conversion
    extend ActiveModel::Naming
    extend ActiveModel::Translation
    include ActiveModel::Validations
    include ActiveModel::AttributeMethods

    include MailForm::Validators

    def initialize(attributes = {})
      attributes.each do |attr, value|
        self.send("#{attr}=", value)
      end unless attributes.blank?
    end

    # 1) Add callbacks behavior
    extend ActiveModel::Callbacks

    # 2) Define the callbacks. The line below will create both before_deliver
    # and after_deliver callbacks with the same semantics as in Active Record
    define_model_callbacks :deliver

    # 3) Change deliver to run the callbacks
    def deliver
      if valid?
        _run_deliver_callbacks do
          MailForm::Notifier.contact(self).deliver
        end
      else
        false
      end
    end

    def persisted?
      false
    end

    # 1) Define a class inheritable attribute named _attributes
    # Let's use underscore to mark this method as internal to our gem
    class_attribute :_attributes
    self._attributes = []

    attribute_method_prefix 'clear_'
    attribute_method_suffix '?'

    def self.attributes(*names)
      attr_accessor *names
      define_attribute_methods names

      # 2) Add declared attributes to the list
      self._attributes += names
    end

    # 3) Create the attributes hash by iterating the list
    def attributes
      self._attributes.inject({}) do |hash, attr|
        hash[attr.to_s] = send(attr)
        hash
      end
    end

    protected

    def clear_attribute(attribute)
      send("#{attribute}=", nil)
    end

    def attribute?(attribute)
      send(attribute).present?
    end
  end
end