#---
# Excerpted from "Crafting Rails Applications",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit http://www.pragmaticprogrammer.com/titles/jvrails for more book information.
#---
require 'test_helper'

class HttpCacheResponderTest < ActionController::TestCase
  tests UsersController
  def setup
    @request.accept = "application/xml"
    ActionController::Base.perform_caching = true
    # Create two resources to be used in :index actions
    User.create(:name => "First", :updated_at => Time.utc(2009))
    User.create(:name => "Second", :updated_at => Time.utc(2008))
  end

  def test_responds_with_last_modified_using_the_latest_timestamp
    get :index
    assert_equal Time.utc(2009).httpdate, @response.headers["Last-Modified"]
    assert_match '<?xml version="1.0" encoding="UTF-8"?>', @response.body
    assert_equal 200, @response.status
  end

  def test_returns_not_modified_if_request_is_still_fresh
    @request.env["HTTP_IF_MODIFIED_SINCE"] = Time.utc(2009, 6).httpdate
    get :index
    assert_equal 304, @response.status
    assert @response.body.blank?
  end

  def test_returns_ok_with_last_modified_if_request_is_not_fresh
    @request.env["HTTP_IF_MODIFIED_SINCE"] = Time.utc(2008, 6).httpdate
    get :index
    assert_equal Time.utc(2009).httpdate, @response.headers["Last-Modified"]
    assert_match '<?xml version="1.0" encoding="UTF-8"?>', @response.body
    assert_equal 200, @response.status
  end
end