#---
# Excerpted from "Crafting Rails Applications",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit http://www.pragmaticprogrammer.com/titles/jvrails for more book information.
#---
require 'test_helper'

class SqlMetricsTest < ActiveSupport::TestCase
  test 'can ignore notifications when specified' do
    SqlMetrics.mute! do
      assert SqlMetrics.mute?
      ActiveSupport::Notifications.instrument "sql.any_orm" do
        sleep(0.001) # sleep for 1000 microseconds
      end

      SqlMetrics.finish!
    end

    assert !SqlMetrics.mute?
    assert_equal 0, SqlMetrics::Metric.count
  end

  test "any sql. notification is saved in the mongo database" do
    payload = { "sql" => "SELECT * FROM foo" }

    ActiveSupport::Notifications.instrument "sql.any_orm", payload do
      sleep(0.001) # sleep for 1000 microseconds
    end

    SqlMetrics.finish!

    metric = SqlMetrics::Metric.first
    assert_equal 1, SqlMetrics::Metric.count
    assert_equal "sql.any_orm", metric.name
    assert_equal payload, metric.payload

    assert metric.duration
    assert metric.instrumenter_id
    assert metric.started_at
    assert metric.created_at
  end
end